unit QryThreadSafeU;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  DB, DBTables, StdCtrls, Grids, DBGrids, ExtCtrls;

type
  TForm1 = class(TForm)
    Session1: TSession;
    Query1: TQuery;
    Query2: TQuery;
    DataSource1: TDataSource;
    DataSource2: TDataSource;
    DBGrid1: TDBGrid;
    DBGrid2: TDBGrid;
    C1: TCheckBox;
    C2: TCheckBox;
    Timer1: TTimer;
    CreateThreadBtn: TButton;
    BeginThreadBtn: TButton;
    TThreadBtn: TButton;
    procedure CreateThreadBtnClick(Sender: TObject);
    procedure BeginThreadBtnClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure TThreadBtnClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  end;

  TQueryThread = class(TThread)
  private
    FQuery: TQuery;
  protected
    constructor Create(Query: TQuery);
    procedure Execute; override;
    procedure DoIt;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

var
  ThreadID1, ThreadID2: DWord;
  OKToContinue: Boolean;

{ Should take Param: Pointer, but an object reference is a pointer anyway }
{ Note the stdcall requirement for CreateThread }
function CreateThreadRoutine(Query: TQuery): Integer; stdcall;
begin
  Result := 0;
  try
    Query.Open;
  finally
    Query.Tag := 0;
  end;
end;

{ Note no stdcall for BeginThread }
function BeginThreadRoutine(Param: Pointer): Integer;
begin
  Result := 0;
  try
    TQuery(Param).Open;
  finally
    TQuery(Param).Tag := 0;
  end;
end;

constructor TQueryThread.Create(Query: TQuery);
begin
  inherited Create(False);
  FreeOnTerminate := True;
  FQuery := Query;
end;

procedure TQueryThread.Execute;
begin
  Synchronize(DoIt);
end;

procedure TQueryThread.DoIt;
begin
  try
    FQuery.Open;
  finally
    FQuery.Tag := 0;
  end;
end;

procedure TForm1.CreateThreadBtnClick(Sender: TObject);
begin
  Query1.Close;
  Query1.Tag := 1;
  Query2.Close;
  Query2.Tag := 1;
  { CreateThread(
      Security Attributes - usually nil,
      Initial thread stack size, 0 means process's stack size,
      Address of thread routine,
      32-bit user-defined parameter,
      Thread flags, eg. Create_Suspended,
      Thread ID variable); }
  CreateThread(nil, 0, @CreateThreadRoutine,
    Pointer(Query1), 0, ThreadID1);
  CreateThread(nil, 0, @CreateThreadRoutine,
    Pointer(Query2), 0, ThreadID2);
end;

procedure TForm1.BeginThreadBtnClick(Sender: TObject);
begin
  Query1.Close;
  Query1.Tag := 1;
  Query2.Close;
  Query2.Tag := 1;
  { BeginThread(
      Security Attributes - usually nil,
      Initial thread stack size, 0 means process's stack size,
      Thread routine,
      32-bit user-defined parameter,
      Thread flags, eg. Create_Suspended,
      Thread ID variable); }
  BeginThread(nil, 0, BeginThreadRoutine,
    Pointer(Query1), 0, ThreadID1);
  BeginThread(nil, 0, BeginThreadRoutine,
    Pointer(Query2), 0, ThreadID2);
end;

procedure TForm1.TThreadBtnClick(Sender: TObject);
begin
  Query1.Close;
  Query1.Tag := 1;
  Query2.Close;
  Query2.Tag := 1;
  TQueryThread.Create(Query1);
  TQueryThread.Create(Query2);
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  C1.Checked := Query1.Active;
  C2.Checked := Query2.Active;
  OKToContinue := (Query1.Tag = 0) and (Query2.Tag = 0);
  CreateThreadBtn.Enabled := OKToContinue;
  BeginThreadBtn.Enabled := OKToContinue;
  TThreadBtn.Enabled := OKToContinue;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := OKToContinue;
end;

end.
