unit TransBmp1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

const
  idOr = 1;
  idAnd = 2;

type
  TTransBmp1 = class(TGraphicControl)
  private
    FOrBitmap,
    FAndBitmap: TBitmap;
  protected
    function GetBitmap(Index: Integer): TBitmap;
    procedure SetBitmap(Index: Integer; Bmp: TBitmap);
    procedure PaintBmp(Bmp: TBitmap; Mode: TCopyMode);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  published
    property OrBitmap : TBitmap index idOr  read GetBitmap write SetBitmap;
    property AndBitmap: TBitmap index idAnd read GetBitmap write SetBitmap;
  end;

procedure Register;

implementation

constructor TTransBmp1.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOrBitmap := TBitmap.Create;
  FAndBitmap := TBitmap.Create;
  { Set arbitrary default, otherwise it will be 0x0 }
  Height := 40;
  Width := 40;
end;

destructor TTransBmp1.Destroy;
begin
  FOrBitmap.Free;
  FAndBitmap.Free;
  inherited Destroy;
end;

function TTransBmp1.GetBitmap(Index: Integer): TBitmap;
begin
  Result := nil;
  case Index of
    idOr: Result := FOrBitmap;
    idAnd: Result := FAndBitmap;
  end;
end;

procedure TTransBmp1.SetBitmap(Index: Integer; Bmp: TBitmap);
begin
  case Index of
    idOr: FOrBitmap.Assign(Bmp);
    idAnd: FAndBitmap.Assign(Bmp);
  end;
  if Assigned(Bmp) then
  begin
    Height := Bmp.Height;
    Width  := Bmp.Width;
  end;
  Invalidate;
end;

procedure TTransBmp1.PaintBmp(Bmp: TBitmap; Mode: TCopyMode);
var
  R: TRect;
begin
  R := Rect(0, 0, Width, Height);
  if Assigned(Bmp) then
    with Canvas do
    begin
      CopyMode := Mode;
      CopyRect(R, Bmp.Canvas, R);
    end;
end;

procedure TTransBmp1.Paint;
begin
  PaintBmp(FAndBitmap, cmSrcAnd);
  PaintBmp(FOrBitmap, cmSrcPaint);
end;

procedure Register;
begin
  RegisterComponents('Samples', [TTransBmp1]);
end;

end.
